/*
 * Copyright (c) 2013-2016 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the view panel for the Quartz scheduler described by the specified
 * connection description configuration object.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.RefreshActiveTab',

    'QDesk.controller.command.scheduler.common.StartScheduler',
    'QDesk.controller.command.scheduler.common.PauseScheduler',
    'QDesk.controller.command.scheduler.common.StopScheduler',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',

    'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab',

    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',

    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',

    'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab',
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',

    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',

    'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab',

    'QDesk.enums.ProductEdition',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProduct'
  ],

  /*
   * Required connection ID configuration attribute.
   */
  connectionId : undefined,

  /**
   * Required scheduler info object with the following structure:
   * {
   *   type : QUARTZ
   *   version : {
   *     major : 1,
   *     minor : 2,
   *     maintenance : 3
   *   }
   *
   *   publicApiInfo : {
   *     detected : true/false,
   *     loadedCorrectly : true/false,
   *     compatible : true/false,
   *     ready : true / false,
   *     version : {
   *       major : 1,
   *       minor : 2,
   *       maintenance : 3,
   *       qualifier : foobar
   *     },
   *   },
   * }
   */
  schedulerInfo : undefined,

  /**
   * Required QuartzDesk JVM Agent info object with the following structure:
   * {
   *   detected : true/false,
   *   compatibleVersion : true/false,
   *   compatibleEdition : true/false,
   *   compatible : true/false,
   *   version : {
   *      major : 1,
   *      minor : 2,
   *      maintenance : 3,
   *      qualifier : foobar
   *   },
   *   edition : Lite/Standard/Enterprise
   *  }
   */
  jvmAgentInfo : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: schedulerInfo' );

    if ( !config.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: jvmAgentInfo' );

    //debugger;

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var schedulerTab = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTab( me.connectionId );

    // if there is no tab panel for the scheduler, create a new one and add it to the mainTabPanel, otherwise
    // just activate the existing tab panel
    if ( schedulerTab == null )
    {
      QDesk.util.UxUtils.maskBody(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.schedulerActionAgentInfo,
        params : {
          connectionId : me.connectionId
        },
        callback : function ( opts, success, resp )
        {
          QDesk.util.UxUtils.unmaskBody();

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              /*
               * Add missing JVM agent info attributes that are not, for performance reasons, available
               * in the ConnectionTreeNode and passed in the jvmAgentInfo object on the input.
               */
              me.jvmAgentInfo.nameAndVersion = json.data.nameAndVersion;
              me.jvmAgentInfo.displayVersion = json.data.displayVersion;

              me.jvmAgentInfo.timeZone = json.data.timeZone;
              me.jvmAgentInfo.availableTimeZones = json.data.availableTimeZones;
              me.jvmAgentInfo.configuration = json.data.configuration;

              me._doChecksAndOpenSchedulerTabPanel( mainTabPanel );
            }
            else
            {
              // Server side error
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
          else
          {
            // Ajax error
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.errorMsg' ) );
          }
        }
      } );
    }
    else
    {
      // make the panel active
      mainTabPanel.setActiveTab( schedulerTab );
    }
  },


  _doChecksAndOpenSchedulerTabPanel : function ( mainTabPanel )
  {
    var me = this;

    if ( me.jvmAgentInfo.detected )
    {
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.enums.ProductEdition.LITE )
      {
        // For QuartzDesk Lite edition we SKIP QuartzDesk JVM Agent version check!
        me._openSchedulerTabPanel( mainTabPanel );
      }
      else
      {
        // For QuartzDesk Standard and Enterprise editions we PERFORM QuartzDesk JVM Agent version and edition check!

        /* compatible version and edition */
        if ( me.jvmAgentInfo.compatible )
        {
          // *** agent version and edition are compatible ***
          me._checkPublicApi( function ()
          {
            me._openSchedulerTabPanel( mainTabPanel );
          } );
        }
        else
        {
          if ( me.jvmAgentInfo.compatibleEdition )
          {
            if ( me.jvmAgentInfo.compatibleVersion )
            {
              // *** agent edition and version are compatible ***
              me._checkPublicApi( function ()
              {
                me._openSchedulerTabPanel( mainTabPanel );
              } );
            }
            else
            {
              // *** agent version is not compatible ***
              var productVersion = QDesk.util.license.LicensedProduct.productVersion;

              QDesk.util.UxUtils.showWarningDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleVersionMsg',
                      QDesk.util.FormatUtils.formatProductVersion( me.jvmAgentInfo.version ),
                      QDesk.util.FormatUtils.formatProductVersion( productVersion )
                  ),
                  function ()
                  {
                    me._openSchedulerTabPanel( mainTabPanel );
                  } );
            }
          }
          else
          {
            // *** agent edition is not compatible ***
            var productEdition = QDesk.util.license.LicensedProduct.productEdition;

            QDesk.util.UxUtils.showWarningDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentIncompatibleEditionMsg',
                    me.jvmAgentInfo.edition,
                    productEdition
                ),
                function ()
                {
                  me._openSchedulerTabPanel( mainTabPanel );
                } );
          }
        }
        /* End of QuartzDesk JVM Agent vs QuartzDesk app version check */
      }
    }
    else
    {
      //
      // JVM agent has not been detected
      //

      // Lite edition
      if ( QDesk.util.license.LicensedProduct.productEdition == QDesk.enums.ProductEdition.LITE )
      {
        // For Lite editions we IGNORE that the QuartzDesk JVM Agent has not been detected!
        me._openSchedulerTabPanel( mainTabPanel );
      }
      else
      {
        // Standard and Enterprise editions
        if ( QDesk.ApplicationConfig.get( 'warning.jvmAgentNotDetected' ) )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.jvmAgentNotDetectedMsg' ),
              function ()
              {
                me._openSchedulerTabPanel( mainTabPanel );
              } );
        }
        else
        {
          me._openSchedulerTabPanel( mainTabPanel );
        }
      }
    }
  },

  /**
   * Checks whether the QuartzDesk Public API Library has been detected, loaded correctly and has a version that is
   * compatible with the installed QuartzDesk JVM Agent. When done, this method executes the specified callback
   * function.
   * Please note that this function is called iff the agent has been detected and is compatible.
   *
   * @param callback a callback function.
   * @private
   */
  _checkPublicApi : function ( callback )
  {
    var me = this;

    var publicApiInfo = me.schedulerInfo.publicApiInfo;
    if ( publicApiInfo == null )
    {
      // should never happen when the agent is installed
    }
    else
    {
      if ( publicApiInfo.ready )
      {
        // *** public API has been detected, is loaded correctly and it has a compatible version
        callback.apply( this );
      }
      else
      {
        if ( !publicApiInfo.detected )
        {
          QDesk.util.UxUtils.showWarningDialog(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiNotDetectedMsg'
              ),
              callback );
        }
        else if ( !publicApiInfo.loadedCorrectly )
        {
          QDesk.util.UxUtils.showWarningDialog(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiNotLoadedCorrectlyMsg'
              ),
              callback );
        }
        else if ( !publicApiInfo.compatibleVersion )
        {
          QDesk.util.UxUtils.showWarningDialog(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.publicApiIncompatibleVersionMsg',
                  QDesk.util.FormatUtils.formatProductVersion( publicApiInfo.version ),
                  QDesk.util.FormatUtils.formatProductVersion( me.jvmAgentInfo.version )
              ),
              callback );
        }
        else
        {
          Ext.Error.raise( 'Unexpected QuartzDesk Public API Library configuration.' );
        }
      }
    }
  },


  /**
   * Opens the Quartz scheduler tab and adds it to the specified main tab panel.
   * @param mainTabPanel the main tab panel.
   */
  _openSchedulerTabPanel : function ( mainTabPanel )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView.maskMsg' ) );

    // deferred so that the mask message gets displayed
    Ext.defer( function ()
    {
      var me = this;

      // open a new Quartz scheduler tab panel and add it to the main tab panel
      var schedulerTab = mainTabPanel.add( {
        xtype : 'QuartzSchedulerTab',
        itemId : QDesk.controller.command.connection.ConnectionUtils.getSchedulerTabItemId( me.connectionId ),
        title : QDesk.controller.command.connection.ConnectionUtils.getConnectionPanelTitle( me.connectionId ),
        qdesk : {
          connectionId : me.connectionId,
          schedulerInfo : me.schedulerInfo,
          jvmAgentInfo : me.jvmAgentInfo
        }
      } );

      mainTabPanel.setActiveTab( schedulerTab );

      me._initializeSchedulerTabPanel( schedulerTab, me.connectionId );

      me.unmask();
    }, 100, me );
  },


  /**
   * Initializes the scheduler tab panel by binding all UI panel controls with the logic.
   * @param tabPanel a Quartz scheduler tab panel.
   */
  _initializeSchedulerTabPanel : function ( tabPanel )
  {
    var me = this;

    var refreshButton = tabPanel.down( '#schedulerTabToolBar > #refreshButton' );
    refreshButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var startButton = tabPanel.down( '#schedulerTabToolBar > #startSchedulerButton' );
    startButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StartScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var pauseButton = tabPanel.down( '#schedulerTabToolBar > #pauseSchedulerButton' );
    pauseButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.PauseScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    var stopButton = tabPanel.down( '#schedulerTabToolBar > #stopSchedulerButton' );
    stopButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.scheduler.common.StopScheduler', {
        connectionId : me.connectionId
      } ).execute();
    } );

    me._initializeSubTabs( tabPanel );
  },


  /**
   * Initializes sub tabs of the specified scheduler tab.
   * @param tabPanel a scheduler tab (the main tab).
   */
  _initializeSubTabs : function ( tabPanel )
  {
    var me = this;

    var refreshButton = tabPanel.down( '#schedulerTabToolBar > #refreshButton' );

    var jobsTab = tabPanel.down( '#jobsTab' );
    jobsTab.on( 'activate', function ( jobsTab )
    {
      /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !jobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var executingJobsTab = tabPanel.down( '#executingJobsTab' );
    executingJobsTab.on( 'activate', function ( executingJobsTab )
    {
      /* If the executing jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !executingJobsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var plannedJobExecutionsTab = tabPanel.down( '#plannedJobExecutionsTab' );
    plannedJobExecutionsTab.on( 'activate', function ( plannedJobExecutionsTab )
    {
      /* If the planned job executions tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !plannedJobExecutionsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializePlannedJobExecutionsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // disable the Refresh button in the lower toolbar
      refreshButton.setDisabled( true );
    } );


    var triggersTab = tabPanel.down( '#triggersTab' );
    triggersTab.on( 'activate', function ( triggersTab )
    {
      /* If the triggers tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !triggersTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var infoTab = tabPanel.down( '#infoTab' );
    infoTab.on( 'activate', function ( infoTab )
    {
      /* If the info tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !infoTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var schedulerExecHistoryTab = tabPanel.down( '#schedulerExecHistoryTab' );
    schedulerExecHistoryTab.on( 'activate', function ( schedulerExecHistoryTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerExecHistoryTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();


        Ext.create( 'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var schedulerStatisticsTab = tabPanel.down( '#schedulerStatisticsTab' );
    schedulerStatisticsTab.on( 'activate', function ( schedulerStatisticsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerStatisticsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var schedulerNotificationsTab = tabPanel.down( '#schedulerNotificationsTab' );
    schedulerNotificationsTab.on( 'activate', function ( schedulerNotificationsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerNotificationsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    var schedulerJobChainsTab = tabPanel.down( '#schedulerJobChainsTab' );
    schedulerJobChainsTab.on( 'activate', function ( schedulerJobChainsTab )
    {
      /* If the tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
      if ( !schedulerJobChainsTab.getInitialized() )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();

        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }

      // enable the Refresh button in the lower toolbar
      refreshButton.setDisabled( false );
    } );


    /* Initially selected tab (this could be made configurable...) */
    tabPanel.setActiveTab( jobsTab );

    // setActiveTab does not fire the activate event if the tab is already active so we need to
    // simulate the active event...

    /* If the jobs tab has not been initialized yet, then initialize it (lazy initialization upon first tab activation) */
    if ( !jobsTab.getInitialized() )
    {
      Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab', {
        connectionId : me.connectionId
      } ).execute();
    }

    Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshActiveTab', {
      connectionId : me.connectionId
    } ).execute();
  }
} );
